﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace AztecDemoConsole
{
    public class AztecReader
    {

#if x86

        [DllImport(@"AztecCodeReaderx86.dll")]
        private static extern IntPtr recognizeY(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx86.dll")]
        private static extern IntPtr recognizeBGRA(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx86.dll")]
        private static extern IntPtr recognizePhotoBGRA(byte[] str, int width, int height);

#elif x64

        [DllImport(@"AztecCodeReaderx64.dll")]
        private static extern IntPtr recognizeY(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx64.dll")]
        private static extern IntPtr recognizeBGRA(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx64.dll")]
        private static extern IntPtr recognizePhotoBGRA(byte[] str, int width, int height);
#endif


        private AztecReader() { }

        public static string Recognize(string filePath)
        {
            var image = Image.FromFile(filePath);
            var result = Recognize(image);
            return result;
        }

        public static string Recognize(Image image)
        {
            var bitmap = new Bitmap(image);
            var bytes = GetPixels(bitmap);
            var resultPtr = recognizePhotoBGRA(bytes, bitmap.Size.Width, bitmap.Size.Height);
            var result = Marshal.PtrToStringAnsi(resultPtr);


            return result;
        }
        public static byte[] GetPixels(Bitmap bitmap)
        {
            if (bitmap.PixelFormat.Equals(PixelFormat.Format32bppArgb))
            {
                var argbData = new byte[bitmap.Width * bitmap.Height * 4];
                var bd = bitmap.LockBits(new Rectangle(0, 0, bitmap.Width, bitmap.Height), ImageLockMode.ReadOnly, bitmap.PixelFormat);
                Marshal.Copy(bd.Scan0, argbData, 0, bitmap.Width * bitmap.Height * 4);
                bitmap.UnlockBits(bd);
                return argbData;
            }
            throw new Exception("GetPixels error");
        }
    }
}
