﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace AztecDemoWPF
{
    public class AztecReader
    {
#if x86

        [DllImport(@"AztecCodeReaderx86.dll")]
        public static extern IntPtr recognizeY(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx86.dll")]
        public static extern IntPtr recognizeBGRA(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx86.dll")]
        public static extern IntPtr recognizePhotoBGRA(byte[] str, int width, int height);

#elif x64

        [DllImport(@"AztecCodeReaderx64.dll")]
        public static extern IntPtr recognizeY(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx64.dll")]
        public static extern IntPtr recognizeBGRA(byte[] str, int width, int height);

        [DllImport(@"AztecCodeReaderx64.dll")]
        public static extern IntPtr recognizePhotoBGRA(byte[] str, int width, int height);

#endif

        private AztecReader() { }

        public static string RecognizeCamera(string filePath)
        {
            var image = Image.FromFile(filePath);
            var result = RecognizeCamera(image);
            return result;
        }

        public static string RecognizeCamera(Image image)
        {
            var bitmap = new Bitmap(image);
            var bytes = GetPixels(bitmap);
            var resultPtr = recognizeBGRA(bytes, bitmap.Size.Width, bitmap.Size.Height);
            var result = Marshal.PtrToStringAnsi(resultPtr);
            return result;
        }

        public static string RecognizePhoto(string filePath)
        {
            var image = Image.FromFile(filePath);
            var result = RecognizePhoto(image);
            return result;
        }

        public static string RecognizePhoto(Image image)
        {
            var bitmap = new Bitmap(image);
            var bytes = GetPixels(bitmap);
            try
            {
                var resultPtr = recognizePhotoBGRA(bytes, bitmap.Size.Width, bitmap.Size.Height);
                var result = Marshal.PtrToStringAnsi(resultPtr);
                return result;
            }
            catch(Exception ex)
            {
                return string.Empty;
            }
        }



        public static byte[] GetPixels(Bitmap bitmap)
        {
            if (bitmap.PixelFormat.Equals(PixelFormat.Format32bppArgb))
            {
                var argbData = new byte[bitmap.Width * bitmap.Height * 4];
                var bd = bitmap.LockBits(new Rectangle(0, 0, bitmap.Width, bitmap.Height), ImageLockMode.ReadOnly, bitmap.PixelFormat);
                Marshal.Copy(bd.Scan0, argbData, 0, bitmap.Width * bitmap.Height * 4);
                bitmap.UnlockBits(bd);
                return argbData;
            }
            throw new Exception("GetPixels error");
        }
    }
}
