﻿using AForge.Video;
using AForge.Video.DirectShow;
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace AztecDemoWPF
{

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        const int MAX_RESOLUTION = 1024 * 768;

        VideoCaptureDevice LocalWebCam;
        FilterInfoCollection LocalWebCamsCollection;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            LocalWebCamsCollection = new FilterInfoCollection(FilterCategory.VideoInputDevice);

            comboBox.ItemsSource = LocalWebCamsCollection as IEnumerable;

            if (comboBox.Items.Count > 0)
            {
                comboBox.SelectedItem = comboBox.Items[0];
            }
        }

        void Cam_NewFrame(object sender, NewFrameEventArgs eventArgs)
        {
            try
            {
                var result = AztecReader.RecognizeCamera(eventArgs.Frame);
                if (!string.IsNullOrEmpty(result))
                {
                    Debug.WriteLine(result);
                    Application.Current.Dispatcher.BeginInvoke(
                        DispatcherPriority.Background,
                        new Action(() =>
                        {
                            resultTextBlock.Text = result;
                            gridResult.Visibility = Visibility.Visible;
                        }));

                }
                else
                {
                    Debug.WriteLine("not recognized");
                }
                System.Drawing.Image img = (Bitmap)eventArgs.Frame.Clone();

                MemoryStream ms = new MemoryStream();
                img.Save(ms, ImageFormat.Bmp);
                ms.Seek(0, SeekOrigin.Begin);
                BitmapImage bi = new BitmapImage();
                bi.BeginInit();
                bi.StreamSource = ms;
                bi.EndInit();
                bi.Freeze();
                Dispatcher.BeginInvoke(new ThreadStart(delegate
                {
                    frameHolder.Source = bi;
                }));
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex);
            }
        }

        private void StartCamera(int index)
        {
            LocalWebCam = new VideoCaptureDevice(LocalWebCamsCollection[index].MonikerString);
            var resolution = LocalWebCam.VideoCapabilities.Where(x => x.FrameSize.Height * x.FrameSize.Width <= MAX_RESOLUTION).OrderByDescending(x => x.FrameSize.Height * x.FrameSize.Width).FirstOrDefault();
            LocalWebCam.VideoResolution = resolution;

            LocalWebCam.NewFrame += new NewFrameEventHandler(Cam_NewFrame);
            LocalWebCam.Start();
        }

        private void StopCamera()
        {
            if (LocalWebCam != null)
            {
                LocalWebCam.NewFrame -= new NewFrameEventHandler(Cam_NewFrame);
                LocalWebCam.SignalToStop();
            }
        }

        private void comboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            StopCamera();
            StartCamera(((ComboBox)sender).SelectedIndex);
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            StopCamera();
            base.OnClosing(e);
        }

        private void ButtonGridResult_Click(object sender, RoutedEventArgs e)
        {
            gridResult.Visibility = Visibility.Collapsed;
        }

        private void ButtonOpenFile_Click(object sender, RoutedEventArgs e)
        {
            Microsoft.Win32.OpenFileDialog dlg = new Microsoft.Win32.OpenFileDialog();

            dlg.Filter = "PNG Files (*.png)|*.png|JPEG Files (*.jpeg)|*.jpeg|JPG Files (*.jpg)|*.jpg|GIF Files (*.gif)|*.gif";

            Nullable<bool> result = dlg.ShowDialog();

            // Get the selected file name and display in a TextBox 
            if (result == true)
            {
                string filename = dlg.FileName;
                var recognized = AztecReader.RecognizePhoto(filename);
                if (!string.IsNullOrEmpty(recognized))
                {
                    resultTextBlock.Text = recognized;
                    gridResult.Visibility = Visibility.Visible;
                }
            }
        }
    }
}
